<?php
/**
*
* @package phpBB3
* @version $Id$
* @copyright (c) 2005 phpBB Group, sections (c) 2001 ispi of Lincoln Inc
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Base Sportsbook class.
* @package phpBB3
*/
class sportsbook
{
  var $config = array();
  var $teams = array();
  var $leagues = array();
  /**
	* Construct : basic setup
	* @access public
	*/
  
  function __construct()
  {
    global $template;
    
    $this->gen_config(); 
    $this->gen_leagues();
    $this->gen_teams();
    
    return;
  }
	 
	/**
	* Set Sportsbook Configuration
	* @access public
	*/
	
  function gen_config()
  {
    global $db;
    
    $sql = 'SELECT *
		FROM ' . SPORTSBOOK_CONFIG_TABLE;
				
		$result = $db->sql_query($sql, SPORTSBOOK_CACHE_LIMIT);
		while ($row = $db->sql_fetchrow($result))
		{
			$this->config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);
		
    return;
  }
  
  /**
	* Generate Leagues
	* @access public
	*/
	
  function gen_leagues()
  {
    global $db;
    
    $sql = 'SELECT *
		FROM ' . SPORTSBOOK_LEAGUES_TABLE . '
    ORDER BY league_name';
				
		$result = $db->sql_query($sql, SPORTSBOOK_CACHE_LIMIT);
		while ($row = $db->sql_fetchrow($result))
		{
			$this->leagues[$row['league_id']] = $row['league_name'];
		}
		$db->sql_freeresult($result);
		
    return;
  }
  
  /**
	* Generate Teams
	* @access public
	*/
	
  function gen_teams()
  {
    global $db;
    
    $sql = 'SELECT *
		FROM ' . SPORTSBOOK_TEAMS_TABLE . '
    ORDER BY team_name';
				
		$result = $db->sql_query($sql, SPORTSBOOK_CACHE_LIMIT);
		while ($row = $db->sql_fetchrow($result))
		{
			$this->teams[$row['team_id']] = $row['team_name'];
		}
		$db->sql_freeresult($result);
		
    return;
  }
  
  /**
	* Generate League Teams
	* @access public
	*/
	
  function gen_league_teams()
  {
    global $db;
    
    $teams = array();
    
    $sql = 'SELECT league_id, team_id
		FROM ' . SPORTSBOOK_TEAMS_TABLE . '
    ORDER BY league_id';
				
		$result = $db->sql_query($sql, SPORTSBOOK_CACHE_LIMIT);
		while ($row = $db->sql_fetchrow($result))
		{
			$teams[$row['league_id']][] = $row['team_id'];
		}
		$db->sql_freeresult($result);
		
    return $teams;    
  }
  
  /**
	* Generate League Team Data
	* @access public
	*/
	
  function gen_league_team_data($league_id)
  {
    global $db;
    
    $data = array(
      'league_id' =>  $league_id
    );
    
    $sql = 'SELECT team_id, team_name
		FROM ' . SPORTSBOOK_TEAMS_TABLE . '
		  WHERE ' . $db->sql_build_array('SELECT', $data) . '
    ORDER BY team_name';
				
		$result = $db->sql_query($sql, SPORTSBOOK_CACHE_LIMIT);
		while ($row = $db->sql_fetchrow($result))
		{
			$teams[$row['team_id']] = $row['team_name'];
		}
		$db->sql_freeresult($result);
		
    return $teams;
  }
  
  /**
	* Generate Fixtures
	* @access public
	*/
	function gen_fixtures($f_type = FIXTURES_FUTURE, $predictions_req = 0)
  {
    global $db, $user;
    
    $fixtures = array(
      'pagination' => array(
        'display' =>  false
      )
    );
    
    switch($f_type)
    {
      case FIXTURES_FUTURE:
        if (!$predictions_req)
        {
          $sql_where =  ' WHERE fixture_time > ' . time();
          $sql_order = ' ORDER BY fixture_time ASC';
        }
        else
        {
          $sql_array = array(
            'SELECT'  =>  'f.*, p.pred_home_score, p.pred_away_score',
            
            'FROM'  =>  array(
              SPORTSBOOK_FIXTURES_TABLE => 'f'),
              
            'LEFT_JOIN'  =>  array(
              array(
                'FROM'  =>  array(SPORTSBOOK_USER_PREDICTIONS_TABLE => 'p'),
                'ON'  =>  'f.fixture_id = p.fixture_id AND p.user_id = ' . (int) $predictions_req)
            ),
              
            'WHERE' =>  'f.fixture_time > ' . time(),
            
            'ORDER_BY'  =>  'f.fixture_time ASC'
          );
        }
      break;
      
      case FIXTURES_OUTSTANDING:
        $sql_where =  ' WHERE fixture_time < ' . time() . ' AND (home_score = ' . FIXTURES_OUTSTANDING . ' OR away_score = ' . FIXTURES_OUTSTANDING . ')';
        $sql_order = ' ORDER BY fixture_time ASC';
      break;
      //
      // need limiter here
      //
      case FIXTURES_PAST:
        if (!$predictions_req)
        {
          $sql_where =  ' WHERE fixture_time < ' . time() . ' AND (home_score != ' . FIXTURES_OUTSTANDING . ' AND away_score != ' . FIXTURES_OUTSTANDING . ')';
          $sql_order = ' ORDER BY fixture_time DESC';
        }
        else
        {
          $sql_array = array(
            'SELECT'  =>  'f.*, p.pred_home_score, p.pred_away_score',
            
            'FROM'  =>  array(
              SPORTSBOOK_FIXTURES_TABLE => 'f'),
              
            'LEFT_JOIN'  =>  array(
              array(
                'FROM'  =>  array(SPORTSBOOK_USER_PREDICTIONS_TABLE => 'p'),
                'ON'  =>  'f.fixture_id = p.fixture_id AND p.user_id = ' . (int) $predictions_req)
            ),
              
            'WHERE' =>  'f.fixture_time < ' . time() . ' AND (f.home_score != ' . FIXTURES_OUTSTANDING . ' AND f.away_score != ' . FIXTURES_OUTSTANDING . ')',
            
            'ORDER_BY'  =>  'f.fixture_time DESC'
          );
        }
        
        $start = request_var('start', 0);
        //
        // get total for pagination
        //
        $sql = 'SELECT COUNT(fixture_id) as total_fixtures
          FROM ' . SPORTSBOOK_FIXTURES_TABLE . '
          WHERE fixture_time < ' . time() . ' 
            AND (home_score != ' . FIXTURES_OUTSTANDING . ' 
            AND away_score != ' . FIXTURES_OUTSTANDING . ')';
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        
        $fixtures['pagination']['total_fixtures'] = $total_fixtures = $row['total_fixtures'];
        $fixtures['pagination']['limit'] = $fixture_limit = 20;
        $fixtures['pagination']['start'] = $start = request_var('start', 0);
        $fixtures['pagination']['display'] = true;
        $fixtures['pagination']['display_info'] = $user->lang['PAGINATION_DISPLAYING_ALL_RESULTS'];
         
      break;
      
      default :
         $sql_where =  ' WHERE fixture_time > ' . time();
         $sql_order = ' ORDER BY fixture_time ASC';
      break;
    }
    
    $fixtures['days'] = array();
    $fixtures['leagues'] = array();
    $fixtures['events'] = array();
    $fixtures['dates'] = array();
    $fixtures['fixture_list'] = array();
    
    $zone_offset = ($user->data['user_timezone'] + $user->data['user_dst']) * 3600;
    
    if (!$predictions_req)
    {
      $sql = 'SELECT *
  		FROM ' . SPORTSBOOK_FIXTURES_TABLE . $sql_where . $sql_order;
		}
		else
		{
      $sql = $db->sql_build_query('SELECT', $sql_array);
    }
				
		$result = ($fixtures['pagination']['display']) ? $db->sql_query_limit($sql, $fixture_limit, $start) : $db->sql_query($sql);
		
		while ($row = $db->sql_fetchrow($result))
		{
		  $day = gmdate("Ymd", ($row['fixture_time'] + $zone_offset));
		  $date = gmdate("d-F-Y", ($row['fixture_time'] + $zone_offset));

		  // has this day been added to the list
		  if(!in_array($day, $fixtures['days']))
			{
        $fixtures['days'][] = $day;
        $fixtures['leagues'][$day] = array();
        $fixtures['fixture_list'][$day] = array();
        $fixtures['dates'][$day] = $date;
      }
      // has this league been added to the list
      if(!in_array($row['league_id'], $fixtures['leagues'][$day]))
      {
        $fixtures['leagues'][$day][] = $row['league_id'];
        $fixtures['fixture_list'][$day][$row['league_id']] = array();
      }
      // build the events array
      $fixtures['events'][$day][$row['league_id']][] = array(
        'fixture_id'  =>  $row['fixture_id'],
        'home_team_id' => $row['home_team'],
        'home_team_name' => $this->teams[$row['home_team']],
        'home_team_score' => $row['home_score'], 
        'away_team_id' => $row['away_team'],
        'away_team_name' => $this->teams[$row['away_team']],
        'away_team_score' => $row['away_score'],
        'pred_home_score' =>  (isset($row['pred_home_score'])) ? $row['pred_home_score'] : PREDICTION_NOT_REQUIRED,
        'pred_away_score' =>  (isset($row['pred_away_score'])) ? $row['pred_away_score'] : PREDICTION_NOT_REQUIRED,
        'kickoff' =>  gmdate("H:i", ($row['fixture_time'] + $zone_offset))  
      );
      $fixtures['fixture_list'][$day][$row['league_id']][] = $row['fixture_id'];
		}
		
		$db->sql_freeresult($result);
		
    return $fixtures;
  }
  
  /**
	* Generate Fixtures List
	* @access public
	*/
	
  function build_fixture_list($fixture_data, $add_reqs = array())
  {
    global $template, $user;
    
    $fixture_days = $fixture_data['days'];
    
    $url = (isset($add_reqs['url'])) ? $add_reqs['url'] : '';
     
    for ($i = 0; $i < count($fixture_days); $i++)
    {
      $day = $fixture_days[$i];
      $template->assign_block_vars('days', array(
        'DAY' =>  $fixture_data['dates'][$day])
      );
      
      $fixture_leagues =  $fixture_data['leagues'][$day];
      $league_list = array();
      for ($j = 0; $j < count($fixture_leagues); $j++)
      {
        $league_list[$fixture_leagues[$j]] = $this->leagues[$fixture_leagues[$j]];         
      }
      asort($league_list);
      foreach ($league_list as $league_id => $league_name)
      {
        $events = array();
        $l_fixtures = implode('*', $fixture_data['fixture_list'][$day][$league_id]);
        
        $template->assign_block_vars('days.leagues', array(
        'LEAGUE'  =>  $league_name,
        'U_DELETE'  =>  $url . '&amp;action=delete_fixture&amp;type=league&amp;l_fixture=' . $l_fixtures)
        );
        // build events
        $events = $fixture_data['events'][$day][$league_id];
        // sort the events by kickoff time
        $events = $this->subval_sort($events, 'kickoff');
        
        for ($k = 0; $k < count($events); $k++)
        {
          $home_prediction = BOX_UNSELECTED;
          $away_prediction = BOX_UNSELECTED;
          
          if ($events[$k]['pred_home_score'] != PREDICTION_NOT_REQUIRED)
          {
            $pred_home = $h_box_value = $home_prediction = $events[$k]['pred_home_score'];
            $pred_away = $a_box_value = $away_prediction = $events[$k]['pred_away_score'];
            
            $home = $events[$k]['home_team_score'];
            $away = $events[$k]['away_team_score'];
            
            $h_box_value = $home_prediction = $events[$k]['pred_home_score'];
            $a_box_value = $away_prediction = $events[$k]['pred_away_score'];
            
            $points_gained = $this->calc_prediction_success($pred_home, $pred_away, $home, $away);
          }
          else
          {
            $h_box_value = $events[$k]['home_team_score'];
            $a_box_value = $events[$k]['away_team_score'];
            
            $points_gained = '-';
          }
          $homebox_details = array(
            'name'  =>  'homescore[' . $events[$k]['fixture_id'] . ']',
            'value' =>  $h_box_value
          );
          $awaybox_details = array(
            'name'  =>  'awayscore[' . $events[$k]['fixture_id'] . ']',
            'value' =>  $a_box_value
          );
          $template->assign_block_vars('days.leagues.events', array(
            'HOME_TEAM' =>  $events[$k]['home_team_name'],
            'AWAY_TEAM' =>  $events[$k]['away_team_name'],
            'HOME_TEAM_SCORE' =>  $events[$k]['home_team_score'],
            'AWAY_TEAM_SCORE' =>  $events[$k]['away_team_score'],
            'HOME_PREDICTION' =>  ($home_prediction != BOX_UNSELECTED) ? $home_prediction : '',
            'AWAY_PREDICTION' =>  ($away_prediction != BOX_UNSELECTED) ? $away_prediction : '',
            'POINTS_GAINED' =>  $points_gained,
            'POINTS_POSITIVE' =>  ($points_gained > 0) ? true : false,
            'KICKOFF' =>  $events[$k]['kickoff'],
            'U_DELETE'  =>  $url . '&amp;action=delete_fixture&amp;type=single&amp;fixture=' . $events[$k]['fixture_id'],
            'HOMESCORE_BOX' =>  $this->make_scorebox($homebox_details),
            'AWAYSCORE_BOX' =>  $this->make_scorebox($awaybox_details))
          );
        }
      } 
    }
    
    if ($fixture_data['pagination']['display'])
    {
      $total_fixtures = $fixture_data['pagination']['total_fixtures'];
      $limit = $fixture_data['pagination']['limit'];
      $start = $fixture_data['pagination']['start'];
      $display_info = $fixture_data['pagination']['display_info'];
      
      $template->assign_vars(array(
        'PAGINATION'  =>  generate_pagination($url, $total_fixtures, $limit, $start),
        'PAGE_NUMBER' =>  on_page($total_fixtures, $limit, $start),
        'TOTAL_FIXTURES'  =>  sprintf($user->lang['PAGINATION_TOTAL_FIXTURES'], number_format($total_fixtures)),
        'DISPLAY_INFO'  =>  $display_info)
      );
    }
    
    return;
  }
  
  /**
	* Delete Team
	* @access public
	*/ 
  
  function delete_team($team)
  {
    global $db, $cache;
    
    $fixtures = array();
    
    $sql = 'SELECT fixture_id
		FROM ' . SPORTSBOOK_FIXTURES_TABLE . '
      WHERE home_team = ' . (int) $team . '
      OR away_team = ' . (int) $team;
				
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$fixtures[] = $row['fixture_id'];
		}
		$db->sql_freeresult($result);
		
		$fixtures = array_filter($fixtures);
		if (sizeof($fixtures))
		{
		  $this->delete_fixture(array_filter($fixtures));
		}
    
    $sql = 'DELETE FROM ' . SPORTSBOOK_TEAMS_TABLE . '
      WHERE team_id=' . (int) $team;
    $db->sql_query($sql);
    
    $cache->destroy('sql', SPORTSBOOK_TEAMS_TABLE);
  
    return true;
  }
  
  /**
	* Delete Fixtures
	* @access public
	*/ 
  
  function delete_fixture($id_ary)
  {
    global $db, $cache;
    
    $sql = 'DELETE FROM ' . SPORTSBOOK_FIXTURES_TABLE . '
      WHERE ' . $db->sql_in_set('fixture_id', $id_ary);
    $db->sql_query($sql);
    
    $sql = 'DELETE FROM ' . SPORTSBOOK_USER_PREDICTIONS_TABLE . '
      WHERE ' . $db->sql_in_set('fixture_id', $id_ary);
    $db->sql_query($sql);
    
    $cache->destroy('sql', array(SPORTSBOOK_FIXTURES_TABLE, SPORTSBOOK_USER_PREDICTIONS_TABLE));
    
    return;
  }
  
  /**
	* Make select box
	* @access public
	*/
  
  function make_select_options($options)
	{
	  $s_options = '';
		// Build options
		foreach ($options as $id => $value)
		{
			$s_options .= '<option value="' . $id . '"' . (($value['selected']) ? ' selected="selected"' : '') . '>' . $value['value'] . '</option>';
		}
		return $s_options;
	}
	
	/**
	* Make scorebox
	* @access public
	*/ 
  
  function make_scorebox($boxdetails)
  {
    $scorebox[BOX_UNSELECTED] = array('selected' => false, 'value' => '-');
    $scorebox[BOX_GAP] = array('selected' => false, 'value' => ' ');
    
    for ($i = 0; $i < 10; $i++)
    {
      $scorebox[$i]['selected'] = ($boxdetails['value'] == $i) ? true : false;
      $scorebox[$i]['value'] = $i;
    }
    
    $scorebox = '<select name="' . $boxdetails['name'] . '">' . $this->make_select_options($scorebox) . '</select>';
  
    return $scorebox;
  }
	
	/**
	* Make dateboxes
	* @access public
	*/ 
  
  function make_dateboxes($show_defaults = true, $selections = array())
  {
    global $user;
    
    $show_defaults = (sizeof($selections)) ? false : $show_defaults;
    
    if ($show_defaults)
    {
      $selections = array(
        'DAY' =>  NOT_REQUIRED,
        'MONTH' =>  NOT_REQUIRED,
        'YEAR' =>  NOT_REQUIRED,
        'HOUR' =>  NOT_REQUIRED,
        'MINUTE' =>  NOT_REQUIRED
      );
    }
    
    $zone_offset = ($user->data['user_timezone'] + $user->data['user_dst']) * 3600;
    $rightnow = time() + $zone_offset;
  
    $dateboxes = array();
    $days = array();
    $months = array();
    $years = array();
    $hours = array();
    $minutes = array();
    
    $days[BOX_UNSELECTED] = array('selected' => false, 'value' => $user->lang['DAY']);
    $months[BOX_UNSELECTED] = array('selected' => false, 'value' => $user->lang['MONTH']);
    $years[BOX_UNSELECTED] = array('selected' => false, 'value' => $user->lang['YEAR']);
    $hours[BOX_UNSELECTED] = array('selected' => false, 'value' => $user->lang['HOUR']);
    $minutes[BOX_UNSELECTED] = array('selected' => false, 'value' => $user->lang['MINUTE']);
    
    $days[BOX_GAP] = $months[BOX_GAP] = $years[BOX_GAP] = $hours[BOX_GAP] = $minutes[BOX_GAP] = array('selected' => false, 'value' => ' ');
    
    $month_advance = (intval($this->config['month_advance'])) ? intval($this->config['month_advance']) : 3;
    $month_start = intval(gmdate("n", $rightnow));
    $month_end = $month_start + $month_advance;
    
    $months_view = array(
      1 => $user->lang['JAN'],
      2 => $user->lang['FEB'],
      3 => $user->lang['MAR'],
      4 => $user->lang['APR'],
      5 => $user->lang['MAY'],
      6 => $user->lang['JUN'],
      7 => $user->lang['JUL'],
      8 => $user->lang['AUG'],
      9 => $user->lang['SEP'],
      10 => $user->lang['OCT'],
      11 => $user->lang['NOV'],
      12 => $user->lang['DEC']
    );
    
    for ($i = 1; $i < 32; $i++)
    {
      $days[$i]['selected'] = ($show_defaults && $i == gmdate("d", $rightnow)) ? true : ($selections['DAY'] == $i) ? true : false;
      $days[$i]['value'] = ($i < 10) ? '0' . $i : $i;
    }
    
    $dateboxes['days'] = '<select name="day">' . $this->make_select_options($days) . '</select>';
    
    for ($i = $month_start; $i < $month_end + 1; $i++)
    {
      $key = ($i < 13) ? $i : $i - 12;
      $months[$key]['selected'] = ($show_defaults && $key == gmdate("m", $rightnow)) ? true : ($selections['MONTH'] == $key) ? true : false;
      $months[$key]['value'] = $months_view[$key];
    }
    
    $dateboxes['months'] = '<select name="month">' . $this->make_select_options($months) . '</select>';
    
    $this_year = gmdate("Y", $rightnow);
    $next_year = gmdate("Y", $rightnow) + 1;
    
    $years[$this_year]['selected'] = ($show_defaults) ? true : ($selections['YEAR'] == $this_year) ? true : false;
    $years[$this_year]['value'] = $this_year;
    
    if ($month_end > 12)
    {
      $years[$next_year]['selected'] = ($selections['YEAR'] == $next_year) ? true : false;
      $years[$next_year]['value'] = $next_year;
    }
    $year_disabled = ($month_end > 12) ? '' : ' disabled="disabled"';
    $hidden_year = ($month_end > 12) ? '' : '<input type="hidden" name="year" value="' . $this_year . '" />';
    $selectname = ($month_end > 12) ? 'year' : '';
    
    $dateboxes['years'] = $hidden_year . '<select name="' . $selectname . '"' . $year_disabled . '>' . $this->make_select_options($years) . '</select>';
    
    $hourbox_start = intval($this->config['hourbox_start']);
    $hourbox_end = (intval($this->config['hourbox_end'])) ? intval($this->config['hourbox_end']) : 24;
    if ($hourbox_start > $hourbox_end)
    {
      $hourbox_start = 0;
      $hourbox_end = 23;
    }
    
    for ($i = $hourbox_start; $i < ($hourbox_end + 1); $i++)
    {
      $hours[$i]['selected'] = ($selections['HOUR'] == $i) ? true : false;
      $hours[$i]['value'] = ($i < 10) ? '0' . $i : $i;
    }
    
    $dateboxes['hours'] = '<select name="hour">' . $this->make_select_options($hours) . '</select>';
    
    $minute_increment = (intval($this->config['minutebox_increment'])) ? intval($this->config['minutebox_increment']) : 5;
    for ($i = 0; $i < 60; $i = $i + $minute_increment)
    {
      $minutes[$i]['selected'] = ($selections['MINUTE'] == $i) ? true : false;
      $minutes[$i]['value'] = ($i < 10) ? '0' . $i : $i;
    }
    
    $dateboxes['minutes'] = '<select name="minute">' . $this->make_select_options($minutes) . '</select>';
  
    return $dateboxes;
  }
  
  /**
	* Build team options box
	* @access public
	*/ 
  
  function build_team_box($teamlist, $selected = BOX_UNSELECTED)
  {
    global $user;
    
    $team_options = array(
      BOX_UNSELECTED => array('selected' => ($selected == BOX_UNSELECTED || $selected == BOX_GAP) ? true : false, 'value' => $user->lang['SELECT_TEAM']),
      BOX_GAP => array('selected' => false, 'value' => ' ')
    );
        
    foreach ($teamlist as $id => $name)
    {
      $team_options[$id]['value'] = $name;
      $team_options[$id]['selected'] = ($selected == $id) ? true : false;
    }
        
    $team_box = $this->make_select_options($team_options);
  
    return $team_box;
  }
  
  /**
	* Process results
	* @access public
	*/ 
  
  function process_results($scores)
  {
    global $db, $cache;
    
    $fixtures = array();
    
    for ($i = 0; $i < count($scores); $i++)
    {
      // need to update users, leaderboard, league tables etc...
      $sql_ary = array(
        'home_score'  =>  $scores[$i]['home_score'],
        'away_score'  =>  $scores[$i]['away_score']
      );
      
      $sql = 'UPDATE ' . SPORTSBOOK_FIXTURES_TABLE .  '
        SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
        WHERE fixture_id = ' . (int) $scores[$i]['fixture_id'];
        
      $db->sql_query($sql);     
      $fixtures[] = $scores[$i]['fixture_id'];
    }
    
    $sql_ary = array(
      'prediction_processed'  =>  PREDICTION_PROCESSED
    );
    
    $sql = 'UPDATE ' . SPORTSBOOK_USER_PREDICTIONS_TABLE . '
      SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
      WHERE ' . $db->sql_in_set('fixture_id', $fixtures);
      
    $db->sql_query($sql);
    
    $sql_ary = array(
      'config_value'  =>  PREDICTION_CALC_REQUIRED
    );
    
    $sql = 'UPDATE ' . SPORTSBOOK_CONFIG_TABLE . '
       SET ' . $db->sql_build_array('UPDATE', $sql_ary) . "
       WHERE config_name = '" . $db->sql_escape('prediction_calc_required') . "'";
       
    $db->sql_query($sql);
    
    $cache->destroy('sql', array(
      SPORTSBOOK_FIXTURES_TABLE, 
      SPORTSBOOK_USER_PREDICTIONS_TABLE, 
      SPORTSBOOK_CONFIG_TABLE)
    );
    
    return;
  }
  
  /**
	* Sub Value sort array
	* @access public
	*/ 
  
  function subval_sort($a, $subkey)
  {
    $b = $c = array();
    
  	foreach($a as $k => $v)
    {
  		$b[$k] = $v[$subkey];
  	}
  	
  	asort($b);
  	
  	foreach($b as $key => $val)
    {
  		$c[] = $a[$key];
  	}
  	
	  return $c;
  }
  
  /**
	* validate input scores
	* @access public
	*/
  
  function validate_input_scores()
  {
    $scores = array();
    $i_homes = request_var('homescore', array(0));
    $i_aways = request_var('awayscore', array(0));
    
    if (sizeof($i_homes) == sizeof($i_aways))
    {
      $fixture_ids = array_keys($i_homes);
      for ($i =0; $i < count($fixture_ids); $i++)
      {
        if ($i_homes[$fixture_ids[$i]] != BOX_UNSELECTED && $i_aways[$fixture_ids[$i]] != BOX_UNSELECTED && $i_homes[$fixture_ids[$i]] != BOX_GAP && $i_aways[$fixture_ids[$i]] != BOX_GAP)
        {
          $scores[] = array(
            'fixture_id'  =>  $fixture_ids[$i],
            'home_score'  =>  $i_homes[$fixture_ids[$i]],
            'away_score'  =>  $i_aways[$fixture_ids[$i]]
          );
        }
      }
    }
    
    return $scores;
  }
  
  /**
	* Calculate prediction success
	* @access public
	*/
  
  function calc_prediction_success($pred_home, $pred_away, $home, $away)
  {
    $success = PREDICTION_LOSS;
    
    // correct score
    if ($pred_home == $home && $pred_away == $away)
    {
      $success = CORRECT_SCORE;
    }
    // correct result DRAW
    else if ($pred_home == $pred_away && $home == $away)
    {
      $success = CORRECT_RESULT;
    }
    // correct result HOME WIN
    else if ($pred_home > $pred_away && $home > $away)
    {
      $success = CORRECT_RESULT;
    }
    // correct result AWAY WIN
    else if ($pred_home < $pred_away && $home < $away)
    {
      $success = CORRECT_RESULT;
    }
  
    return $success;
  }
  
  /**
	* Update Config
	* @access public
	*/
  
  function update_config($params)
  {
    global $db, $cache;
    
    foreach ($params as $k => $v)
    {
      $sql_arr = array('config_value' => $v);

      $sql = 'UPDATE ' . SPORTSBOOK_CONFIG_TABLE . ' 
        SET ' . $db->sql_build_array('UPDATE', $sql_arr) . "
        WHERE config_name = '" . $k . "'";
      $db->sql_query($sql);
    }
    
    $cache->destroy('sql', SPORTSBOOK_CONFIG_TABLE);
  
    return;
  }
}

?>