<?php
/**
*
* @package phpBB3
* @version $Id$
* @copyright (c) 2005 phpBB Group, sections (c) 2001 ispi of Lincoln Inc
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Sportsbook User class.
* @package phpBB3
*/
class sportsbook_user extends sportsbook
{
  /**
	* Construct : basic setup
	* @access public
	*/
  
  function __construct()
  {
    parent::__construct();
    
    if ($this->config['prediction_calc_required'] == PREDICTION_CALC_REQUIRED)
    {
      $this->calculate_predictions();
    }
    
    return;
  }
  
  /**
	* Retrieve user predictions
	* @access public
	*/
  
  function retrieve_user_predictions($user_id/*, $prediction_type = PREDICTION_NOT_PROCESSED*/)
  {
    global $db;
    
    $predictions = array();
    
    $sql = 'SELECT fixture_id, pred_home_score, pred_away_score
      FROM ' . SPORTSBOOK_USER_PREDICTIONS_TABLE . '
      WHERE user_id = ' . intval($user_id) . ' AND prediction_processed = ' . PREDICTION_NOT_PROCESSED;
    $result = $db->sql_query($sql);
    
		while ($row = $db->sql_fetchrow($result))
		{
			$predictions[$row['fixture_id']] = array(
        'pred_home_score'  =>  $row['pred_home_score'],
        'pred_away_score'  =>  $row['pred_away_score']
      );
		}
		$db->sql_freeresult($result);
    
    return $predictions;
  }
  
  /**
	* Process predictions
	* @access public
	*/ 
  
  function process_predictions($scores)
  {
    global $db, $cache, $user;
    
    $existing_ids = array_keys($existing_predictions = $this->retrieve_user_predictions($user->data['user_id']));
    
    $times = array();
    $fixture_set = array();
    
    for ($i = 0; $i < count($scores); $i++)
    {
      $fixture_set[] = $scores[$i]['fixture_id'];
    }
       
    // need to prevent an update if fixture time has expired
    $sql = 'SELECT fixture_time, fixture_id 
    FROM ' . SPORTSBOOK_FIXTURES_TABLE . '
      WHERE ' . $db->sql_in_set('fixture_id', $fixture_set);
    $result = $db->sql_query($sql);
    
    while ($row = $db->sql_fetchrow($result))
    {
      $fixture_id = $row['fixture_id'];
      $times[$fixture_id] = $row['fixture_time'];
    }
    
    $sql_multi_ary = array();
        
    for ($i = 0; $i < count($scores); $i++)
    {
      $cnt_fixture = $scores[$i]['fixture_id'];
      $f_time = $times[$cnt_fixture];
      $timenow = time();
      
      if ($f_time > $timenow)
      {
        if (in_array($cnt_fixture, $existing_ids))
        {
          if ($scores[$i]['home_score'] != $existing_predictions[$cnt_fixture]['pred_home_score'] || $scores[$i]['away_score'] != $existing_predictions[$cnt_fixture]['pred_away_score'])
          {
            $sql_ary = array(
              'pred_home_score' =>  $scores[$i]['home_score'],
              'pred_away_score' =>  $scores[$i]['away_score']
            );
            
            $sql = 'UPDATE ' . SPORTSBOOK_USER_PREDICTIONS_TABLE . '
              SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
              WHERE fixture_id = ' . (int) $scores[$i]['fixture_id'] . '
                AND user_id = ' . (int) $user->data['user_id'];
            $db->sql_query($sql);
          }
        }
        else
        {
          $sql_multi_ary[] = array(
            'user_id' => $user->data['user_id'], 
            'fixture_id'  =>  $scores[$i]['fixture_id'],
            'pred_home_score'  =>  $scores[$i]['home_score'],
            'pred_away_score'  =>  $scores[$i]['away_score']
          );
        }
      }
    }
    if (sizeof($sql_multi_ary))
    {
      $db->sql_multi_insert(SPORTSBOOK_USER_PREDICTIONS_TABLE, $sql_multi_ary);
    }
    $cache->destroy('sql', SPORTSBOOK_USER_PREDICTIONS_TABLE);
    
    return;
  }
  
  /**
	* Calculate predictions
	* @access public
	*/
  
  function calculate_predictions()
  {
    global $db, $cache;
    
    $prediction_data = array();
    $users = array();
    $predictions = array();
    $l_users = array();
		//
		// get list of fixtures that require calculation
		//
		$sql_arr = array(
      'SELECT'  =>  'p.prediction_id, p.user_id, p.pred_home_score, p.pred_away_score, f.home_score, f.away_score',
      'FROM'  =>  array(
        SPORTSBOOK_USER_PREDICTIONS_TABLE =>  'p',
        SPORTSBOOK_FIXTURES_TABLE =>  'f'  
      ),
      'WHERE' =>  'p.calc_required = ' . PREDICTION_CALC_REQUIRED . ' AND p.prediction_processed = ' . PREDICTION_PROCESSED . ' AND p.fixture_id = f.fixture_id',
      'ORDER_BY' =>  'p.prediction_id'
    );
    $sql = $db->sql_build_query('SELECT', $sql_arr);
    $result = $db->sql_query_limit($sql, $this->config['prediction_calc_limit']);
  	while ($row = $db->sql_fetchrow($result))
  	{
    	$prediction_data[$row['user_id']][] = array(
        'home_score'  =>  $row['home_score'],
        'away_score'  =>  $row['away_score'],
        'pred_home_score' => $row['pred_home_score'],
        'pred_away_score' => $row['pred_away_score'] 
      );
      
      $users[] = $row['user_id'];
      $predictions[] = $row['prediction_id'];
  	}
  	$db->sql_freeresult($result);
  	$users = (sizeof($users)) ? array_unique($users) : array();
  	//
  	// Get current leaderboard data for affected users
  	//
    if (sizeof($predictions))
    {
      $sql = 'SELECT *
        FROM ' . SPORTSBOOK_LEADERBOARD_TABLE . '
        WHERE ' . $db->sql_in_set('user_id', $users);
        
      $result = $db->sql_query($sql);
    	while ($row = $db->sql_fetchrow($result))
    	{
      	$prediction_data[$row['user_id']][LEADERBOARD] = array(
      	  'leaderboard_action' =>  'UPDATE',
          'predictions' =>  $row['predictions'],
          'correct_scores'  =>  $row['correct_scores'],
          'correct_results' =>  $row['correct_results'],
          'points'  =>  $row['points']
        );
        $l_users[] = $row['user_id'];
    	}
    	$db->sql_freeresult($result);
    	//
    	// get list of users who are first time players
    	//
    	foreach ($users as $key => $user_id)
      {
        if (!in_array($user_id, $l_users))
        {
          $prediction_data[$user_id][LEADERBOARD] = array(
            'leaderboard_action'  =>  'INSERT',
            'predictions' =>  0,
            'correct_scores'  =>  0,
            'correct_results' =>  0,
            'points'  =>  0
          );
        }
      }
      unset($user_id);
      //
      // update the leaderboard
      //
      $sql_multi_ary = array();
      foreach ($users as $k => $user_id)
      {
        //
        // calculate the results
        //
        foreach ($prediction_data[$user_id] as $key => $data)
        {
          if ($key != LEADERBOARD)
          {
            $success = $this->calc_prediction_success($data['pred_home_score'], $data['pred_away_score'], $data['home_score'], $data['away_score']);
            switch ($success)
            {
              case CORRECT_SCORE:
                $prediction_data[$user_id][LEADERBOARD]['correct_scores']++;
                $prediction_data[$user_id][LEADERBOARD]['points'] = $prediction_data[$user_id][LEADERBOARD]['points'] + CORRECT_SCORE;
              break;
              
              case CORRECT_RESULT:
                $prediction_data[$user_id][LEADERBOARD]['correct_results']++;
                $prediction_data[$user_id][LEADERBOARD]['points'] = $prediction_data[$user_id][LEADERBOARD]['points'] + CORRECT_RESULT;
              break;
              
              case PREDICTION_LOSS:
                $prediction_data[$user_id][LEADERBOARD]['points'] = $prediction_data[$user_id][LEADERBOARD]['points'] + PREDICTION_LOSS;
              break;
            }
            $prediction_data[$user_id][LEADERBOARD]['predictions']++;
          }
        }
        //
        // Update the db
        //
        if ($prediction_data[$user_id][LEADERBOARD]['leaderboard_action'] == 'UPDATE')
        {
          $sql_arr = array(
            'predictions' =>  $prediction_data[$user_id][LEADERBOARD]['predictions'],
            'correct_scores'  =>  $prediction_data[$user_id][LEADERBOARD]['correct_scores'],
            'correct_results'  =>  $prediction_data[$user_id][LEADERBOARD]['correct_results'],
            'points'  =>  $prediction_data[$user_id][LEADERBOARD]['points']
          );
          
          $sql = 'UPDATE ' . SPORTSBOOK_LEADERBOARD_TABLE . ' 
            SET ' . $db->sql_build_array('UPDATE', $sql_arr) . '
            WHERE user_id = ' . (int) $user_id;
          $db->sql_query($sql);
        }
        else
        {
          $sql_multi_ary[] = array(
            'user_id' =>  $user_id,
            'predictions' =>  $prediction_data[$user_id][LEADERBOARD]['predictions'],
            'correct_scores'  =>  $prediction_data[$user_id][LEADERBOARD]['correct_scores'],
            'correct_results'  =>  $prediction_data[$user_id][LEADERBOARD]['correct_results'],
            'points'  =>  $prediction_data[$user_id][LEADERBOARD]['points']
          ); 
        }
      }
      if (sizeof($sql_multi_ary))
      {
        $db->sql_multi_insert(SPORTSBOOK_LEADERBOARD_TABLE, $sql_multi_ary);
      }
      //
      // Update the prediction calcs
      //
      $sql_arr = array('calc_required' => PREDICTION_CALC_NOT_REQUIRED);
      
      $sql = 'UPDATE ' . SPORTSBOOK_USER_PREDICTIONS_TABLE . ' 
        SET ' . $db->sql_build_array('UPDATE', $sql_arr) . '
        WHERE ' . $db->sql_in_set('prediction_id', $predictions);
      $db->sql_query($sql);
    }
    //
    // update the config if we're finished
    //
    if (count($predictions) < $this->config['prediction_calc_limit'])
    {
      $sql_arr = array('config_value' => PREDICTION_CALC_NOT_REQUIRED);
      
      $sql = 'UPDATE ' . SPORTSBOOK_CONFIG_TABLE . ' 
        SET ' . $db->sql_build_array('UPDATE', $sql_arr) . "
        WHERE config_name = 'prediction_calc_required'";
      $db->sql_query($sql);
    }
    //
    // Delete all cached data
    //
    $cache->destroy('sql', array(
      SPORTSBOOK_USER_PREDICTIONS_TABLE,
      SPORTSBOOK_CONFIG_TABLE,
      SPORTSBOOK_LEADERBOARD_TABLE,
      SPORTSBOOK_FIXTURES_TABLE)
    );
    
    return;
  }
  
  /**
	* Display Leaderboard
	* @access public
	*/
  
  function display_leaderboard($user_id = 0)
  {
    global $db, $user;
    
    $leaderboard = array();
    //
		// get leaderboard
		//
		if ($user_id)
		{
      $sql = 'SELECT * 
        FROM ' . SPORTSBOOK_LEADERBOARD_TABLE . '
        WHERE user_id = ' . (int) $user_id;
      $result = $db->sql_query($sql);
		}
		else
		{
  		$sql_arr = array(
        'SELECT'  =>  'l.*, u.username',
        'FROM'  =>  array(
          SPORTSBOOK_LEADERBOARD_TABLE  =>  'l',
          USERS_TABLE =>  'u'
        ),
        'WHERE' =>  'l.user_id = u.user_id',
        'ORDER_BY' =>  'l.points DESC'
      );
      
      $user_id = $user->data['user_id'];
      $sql = $db->sql_build_query('SELECT', $sql_arr);
      $result = $db->sql_query_limit($sql, $this->config['leaderboard_limit'], 0, SPORTSBOOK_CACHE_LIMIT);
    }
    
  	while ($row = $db->sql_fetchrow($result))
  	{
    	$leaderboard[] = array(
        'user_id'  =>  $row['user_id'],
        'username'  =>  (isset($row['username'])) ? $row['username'] : $user->data['username'],
        'predictions' => $row['predictions'],
        'correct_results' => $row['correct_results'],
        'correct_scores' => $row['correct_scores'],
        'points' => $row['points'],
        'user_hilite' =>  ($user_id == $row['user_id']) ? true : false 
      );
  	}
  	$db->sql_freeresult($result);
    
    return $leaderboard;
  }
}

?>